// Copyright (c) 2008  Future Technology Devices International Ltd.
//
// Module Name
//
//     MainForm.cs
//
// Abstract:
//
//    C# program, to test the FTCSPI.DLL. The FTCSPI.DLL is used to control the FT2232H dual hi-speed device
//    and the FT4232H quad hi-speed device to simulate the Serial Peripheral Interface(SPI) synchronous serial protocol
//    to communicate with SPI devices connected to FT2232H dual hi-speed devices and FT4232H quad hi-speed devices.
//
// Revision History:
//
//   25/08/08    Kirke Ashton    Created.
//
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Threading;

namespace FT2232HSPICSharpNETTestApp
{
    public partial class MainForm : Form
    {
        private const string App_Title = "FT2232/FT4232 SPI Device C# .NET Test Application";

        private const string Dll_Version_Label = "FT2232/FT4232 SPI DLL Version = ";

        private const string Device_Name_Label = "Device Name = ";

        private enum HI_SPEED_DEVICE_TYPES// : uint
        {
            FT2232H_DEVICE_TYPE = 1,
            FT4232H_DEVICE_TYPE = 2
        };

        private const uint FTC_SUCCESS = 0;
        private const uint FTC_DEVICE_IN_USE = 27;

        private const uint MAX_NUM_DEVICE_NAME_CHARS = 100;
        private const uint MAX_NUM_CHANNEL_CHARS = 5;

        private const uint MAX_NUM_DLL_VERSION_CHARS = 10;
        private const uint MAX_NUM_ERROR_MESSAGE_CHARS = 100;

        // To communicate with the 93LC56(2048 word) EEPROM, the maximum frequency the clock can be set is 1MHz 
        private const uint MAX_FREQ_93LC56_CLOCK_DIVISOR = 29;    // equivalent to 1MHz

        private const bool ADBUS3ChipSelect = false;

        private const uint ADBUS2DataIn = 0;

        private const int WRITE_CONTROL_BUFFER_SIZE = 256;
        private const int WRITE_DATA_BUFFER_SIZE = 65536;
        private const int READ_DATA_BUFFER_SIZE = 65536;
        private const int READ_CMDS_DATA_BUFFER_SIZE = 131071;

        private const uint SPIEWENCmdIndex = 0;
        private const uint SPIEWDSCmdIndex = 1;
        private const uint SPIERALCmdIndex = 2;

        private const uint MAX_SPI_93LC56_CHIP_SIZE_IN_WORDS = 128;

        private const uint NUM_93LC56B_CMD_CONTOL_BITS = 11;
        private const uint NUM_93LC56B_CMD_CONTOL_BYTES = 2;

        private const uint NUM_93LC56B_CMD_DATA_BITS = 16;
        private const uint NUM_93LC56B_CMD_DATA_BYTES = 2;


        //**************************************************************************
        //
        // TYPE DEFINITIONS
        //
        //**************************************************************************

        public struct FTC_CHIP_SELECT_PINS
        {
            public bool bADBUS3ChipSelectPinState;
            public bool bADBUS4GPIOL1PinState;
            public bool bADBUS5GPIOL2PinState;
            public bool bADBUS6GPIOL3PinState;
            public bool bADBUS7GPIOL4PinState;
        }

        public struct FTC_INPUT_OUTPUT_PINS
        {
            public bool bPin1InputOutputState;
            public bool bPin1LowHighState;
            public bool bPin2InputOutputState;
            public bool bPin2LowHighState;
            public bool bPin3InputOutputState;
            public bool bPin3LowHighState;
            public bool bPin4InputOutputState;
            public bool bPin4LowHighState;
        }

        public struct FTH_INPUT_OUTPUT_PINS
        {
            public bool bPin1InputOutputState;
            public bool bPin1LowHighState;
            public bool bPin2InputOutputState;
            public bool bPin2LowHighState;
            public bool bPin3InputOutputState;
            public bool bPin3LowHighState;
            public bool bPin4InputOutputState;
            public bool bPin4LowHighState;
            public bool bPin5InputOutputState;
            public bool bPin5LowHighState;
            public bool bPin6InputOutputState;
            public bool bPin6LowHighState;
            public bool bPin7InputOutputState;
            public bool bPin7LowHighState;
            public bool bPin8InputOutputState;
            public bool bPin8LowHighState;
        }

        public struct FTC_LOW_HIGH_PINS
        {
            public bool bPin1LowHighState;
            public bool bPin2LowHighState;
            public bool bPin3LowHighState;
            public bool bPin4LowHighState;
        }

        public struct FTH_LOW_HIGH_PINS
        {
            public bool bPin1LowHighState;
            public bool bPin2LowHighState;
            public bool bPin3LowHighState;
            public bool bPin4LowHighState;
            public bool bPin5LowHighState;
            public bool bPin6LowHighState;
            public bool bPin7LowHighState;
            public bool bPin8LowHighState;
        }

        public struct FTC_INIT_CONDITION
        {
            public bool bClockPinState;
            public bool bDataOutPinState;
            public bool bChipSelectPinState;
            public bool ChipSelectPin;
        }

        public struct FTC_WAIT_DATA_WRITE
        {
            public bool bWaitDataWriteComplete;
            public uint WaitDataWritePin;
            public bool bDataWriteCompleteState;
            public uint DataWriteTimeoutmSecs;
        }

        public struct FTC_HIGHER_OUTPUT_PINS
        {
            public bool bPin1State;
            public bool bPin1ActiveState;
            public bool bPin2State;
            public bool bPin2ActiveState;
            public bool bPin3State;
            public bool bPin3ActiveState;
            public bool bPin4State;
            public bool bPin4ActiveState;
        }

        public struct FTH_HIGHER_OUTPUT_PINS
        {
            public bool bPin1State;
            public bool bPin1ActiveState;
            public bool bPin2State;
            public bool bPin2ActiveState;
            public bool bPin3State;
            public bool bPin3ActiveState;
            public bool bPin4State;
            public bool bPin4ActiveState;
            public bool bPin5State;
            public bool bPin5ActiveState;
            public bool bPin6State;
            public bool bPin6ActiveState;
            public bool bPin7State;
            public bool bPin7ActiveState;
            public bool bPin8State;
            public bool bPin8ActiveState;
        }

        public struct FTC_CLOSE_FINAL_STATE_PINS
        {
            public bool bTCKPinState;
            public bool bTCKPinActiveState;
            public bool bTDIPinState;
            public bool bTDIPinActiveState;
            public bool bTMSPinState;
            public bool bTMSPinActiveState;
        }


        //**************************************************************************
        //
        // FUNCTION IMPORTS FROM FTCI2C DLL
        //
        //**************************************************************************

        // Built-in Windows API functions to allow us to dynamically load our own DLL.
        [DllImportAttribute("ftcspi.dll", EntryPoint = "SPI_GetDllVersion", CallingConvention = CallingConvention.Cdecl)]
        static extern uint GetDllVersion(byte[] pDllVersion, uint buufferSize);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_GetErrorCodeString(string language, uint statusCode, byte[] pErrorMessage, uint bufferSize);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_GetNumHiSpeedDevices(ref uint NumHiSpeedDevices);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_GetHiSpeedDeviceNameLocIDChannel(uint deviceNameIndex, byte[] pDeviceName, uint deviceNameBufferSize, ref uint locationID, byte[] pChannel, uint channelBufferSize, ref uint hiSpeedDeviceType);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_OpenHiSpeedDevice(string DeviceName, uint locationID, string channel, ref IntPtr pftHandle);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_GetHiSpeedDeviceType(IntPtr ftHandle, ref uint hiSpeedDeviceType);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_Close(IntPtr ftHandle);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_CloseDevice(IntPtr ftHandle, ref FTC_CLOSE_FINAL_STATE_PINS pCloseFinalStatePinsData);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_InitDevice(IntPtr ftHandle, uint clockDivisor);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_TurnOnDivideByFiveClockingHiSpeedDevice(IntPtr ftHandle);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_TurnOffDivideByFiveClockingHiSpeedDevice(IntPtr ftHandle);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_SetDeviceLatencyTimer(IntPtr ftHandle, byte timerValue);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_GetDeviceLatencyTimer(IntPtr ftHandle, ref byte timerValue);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_GetHiSpeedDeviceClock(uint ClockDivisor, ref uint clockFrequencyHz);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_GetClock(uint clockDivisor, ref uint clockFrequencyHz);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_SetClock(IntPtr ftHandle, uint clockDivisor, ref uint clockFrequencyHz);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_SetLoopback(IntPtr ftHandle, bool bLoopBackState);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_SetHiSpeedDeviceGPIOs(IntPtr ftHandle, ref FTC_CHIP_SELECT_PINS pChipSelectsDisableStates, ref FTH_INPUT_OUTPUT_PINS pHighInputOutputPinsData);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_GetHiSpeedDeviceGPIOs(IntPtr ftHandle, out FTH_LOW_HIGH_PINS pHighPinsInputData);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_WriteHiSpeedDevice(IntPtr ftHandle, ref FTC_INIT_CONDITION pWriteStartCondition, bool bClockOutDataBitsMSBFirst, bool bClockOutDataBitsPosEdge, uint numControlBitsToWrite, byte[] pWriteControlBuffer, uint numControlBytesToWrite, bool bWriteDataBits, uint numDataBitsToWrite, byte[] pWriteDataBuffer, uint numDataBytesToWrite, ref FTC_WAIT_DATA_WRITE pWaitDataWriteComplete, ref FTH_HIGHER_OUTPUT_PINS pHighPinsWriteActiveStates);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_ReadHiSpeedDevice(IntPtr ftHandle, ref FTC_INIT_CONDITION pReadStartCondition, bool bClockOutControBitsMSBFirst, bool bClockOutControBitsPosEdge, uint numControlBitsToWrite, byte[] pWriteControlBuffer, uint numControlBytesToWrite, bool bClockInDataBitsMSBFirst, bool bClockInDataBitsPosEdge, uint numDataBitsToRead, byte[] pReadDataBuffer, out uint pnumDataBytesReturned, ref FTH_HIGHER_OUTPUT_PINS pHighPinsReadActiveStates);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_ClearDeviceCmdSequence(IntPtr ftHandle);

        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_AddHiSpeedDeviceReadCmd(IntPtr ftHandle, ref FTC_INIT_CONDITION pReadStartCondition, bool bClockOutControBitsMSBFirst, bool bClockOutControBitsPosEdge, uint numControlBitsToWrite, byte[] pWriteControlBuffer, uint numControlBytesToWrite, bool bClockInDataBitsMSBFirst, bool bClockInDataBitsPosEdge, uint numDataBitsToRead, ref FTH_HIGHER_OUTPUT_PINS pHighPinsReadActiveStates);
        
        [DllImportAttribute("ftcspi.dll", CallingConvention = CallingConvention.Cdecl)]
        static extern uint SPI_ExecuteDeviceCmdSequence(IntPtr ftHandle, byte[] pReadCmdSequenceDataBuffer, out uint pnumDataBytesReturned);


        public MainForm()
        {
            InitializeComponent();

            this.Text = App_Title;

            this.CommandSequenceMenuItem.CheckState = CheckState.Unchecked;

            this.ChannelComboBox.Items.Clear();
            this.ChannelComboBox.Items.Add("A");
            this.ChannelComboBox.Items.Add("B");
            this.ChannelComboBox.Text = "A";
            //this.ChannelComboBox.SelectedIndex = -1;
        }

        private uint SPI_EWEN_EWDS_ERAL_93LC56B_Device(IntPtr ftHandle, ref FTC_INIT_CONDITION pWriteStartCondition, uint SPICmdIndex) {
            uint ftStatus = FTC_SUCCESS;
            byte ControlByte1 = 0;
            byte ControlByte2 = 0;
            byte[] WriteControlBuffer = new byte[WRITE_CONTROL_BUFFER_SIZE];
            byte[] WriteDataBuffer = new byte[WRITE_DATA_BUFFER_SIZE];
            FTC_WAIT_DATA_WRITE WaitDataWriteComplete;
            FTH_HIGHER_OUTPUT_PINS HighPinsWriteActiveStates;

            switch (SPICmdIndex) {
                case SPIEWENCmdIndex:
                    ControlByte1 = 0x9F; // set up write enable command
                    break;
                case SPIEWDSCmdIndex:
                    ControlByte1 = 0x87; // set up write disable command
                    break;
                case SPIERALCmdIndex:
                    ControlByte1 = 0x97; // set up erase all command
                    break;
                default:
                    break;
            }

            ControlByte2 = 0xFF;

            WriteControlBuffer[0] = ControlByte1;
            WriteControlBuffer[1] = ControlByte2;

            WaitDataWriteComplete.bWaitDataWriteComplete = false;
            WaitDataWriteComplete.WaitDataWritePin = 0;
            WaitDataWriteComplete.bDataWriteCompleteState = false;
            WaitDataWriteComplete.DataWriteTimeoutmSecs = 0;

            HighPinsWriteActiveStates.bPin1State = this.GPIOH0WriteUseNotUseStateCheckBox.Checked;
            HighPinsWriteActiveStates.bPin1ActiveState = this.GPIOH0OutputLowHighWriteStateCheckBox.Checked;
            HighPinsWriteActiveStates.bPin2State = this.GPIOH1WriteUseNotUseStateCheckBox.Checked;
            HighPinsWriteActiveStates.bPin2ActiveState = this.GPIOH1OutputLowHighWriteStateCheckBox.Checked;
            HighPinsWriteActiveStates.bPin3State = this.GPIOH2WriteUseNotUseStateCheckBox.Checked;
            HighPinsWriteActiveStates.bPin3ActiveState = this.GPIOH2OutputLowHighWriteStateCheckBox.Checked;
            HighPinsWriteActiveStates.bPin4State = this.GPIOH3WriteUseNotUseStateCheckBox.Checked;
            HighPinsWriteActiveStates.bPin4ActiveState = this.GPIOH3OutputLowHighWriteStateCheckBox.Checked;
            HighPinsWriteActiveStates.bPin5State = this.GPIOH4WriteUseNotUseStateCheckBox.Checked;
            HighPinsWriteActiveStates.bPin5ActiveState = this.GPIOH4OutputLowHighWriteStateCheckBox.Checked;
            HighPinsWriteActiveStates.bPin6State = this.GPIOH5WriteUseNotUseStateCheckBox.Checked;
            HighPinsWriteActiveStates.bPin6ActiveState = this.GPIOH5OutputLowHighWriteStateCheckBox.Checked;
            HighPinsWriteActiveStates.bPin7State = this.GPIOH6WriteUseNotUseStateCheckBox.Checked;
            HighPinsWriteActiveStates.bPin7ActiveState = this.GPIOH6OutputLowHighWriteStateCheckBox.Checked;
            HighPinsWriteActiveStates.bPin8State = this.GPIOH7WriteUseNotUseStateCheckBox.Checked;
            HighPinsWriteActiveStates.bPin8ActiveState = this.GPIOH7OutputLowHighWriteStateCheckBox.Checked;

            ftStatus = SPI_WriteHiSpeedDevice(ftHandle, ref pWriteStartCondition, true, false, 11, WriteControlBuffer, 2, false, 0, WriteDataBuffer, 0, ref WaitDataWriteComplete, ref HighPinsWriteActiveStates);

            return ftStatus;
        }

        private uint TestHiSpeedDeviceClock(IntPtr ftHandle)
        {
            uint ftStatus = FTC_SUCCESS;
            uint clockFrequencyHz = 0;

            if ((ftStatus = SPI_GetHiSpeedDeviceClock(0, ref clockFrequencyHz)) == FTC_SUCCESS)
            {
                if ((ftStatus = SPI_TurnOnDivideByFiveClockingHiSpeedDevice(ftHandle)) == FTC_SUCCESS)
                {
                    ftStatus = SPI_GetHiSpeedDeviceClock(0, ref clockFrequencyHz);

                    if ((ftStatus = SPI_SetClock(ftHandle, MAX_FREQ_93LC56_CLOCK_DIVISOR, ref clockFrequencyHz)) == FTC_SUCCESS)
                    {
                        ftStatus = SPI_TurnOffDivideByFiveClockingHiSpeedDevice(ftHandle);

                        ftStatus = SPI_SetClock(ftHandle, MAX_FREQ_93LC56_CLOCK_DIVISOR, ref clockFrequencyHz);
                    }
                }
            }

            return ftStatus;
        }

        private void OKbutton_Click(object sender, EventArgs e)
        {
            uint ftStatus = FTC_SUCCESS;
            uint numHiSpeedDevices = 0; // 32-bit unsigned integer
            uint hiSpeedDeviceIndex = 0;
            uint locationID = 0;
            byte[] byteHiSpeedDeviceName = new byte[MAX_NUM_DEVICE_NAME_CHARS];
            byte[] byteHiSpeedDeviceChannel = new byte[MAX_NUM_CHANNEL_CHARS];
            uint hiSpeedDeviceType = 0;
            string hiSpeedDeviceName = null;
            string hiSpeedChannel = null;
            IntPtr ftHandle = IntPtr.Zero;
            byte timerValue = 0;
            bool bLoopBackState = false;
            FTC_CHIP_SELECT_PINS ChipSelectsDisableStates;
            FTH_INPUT_OUTPUT_PINS HighInputOutputPins;
            FTH_LOW_HIGH_PINS HighPinsInputData;
            FTC_INIT_CONDITION WriteStartCondition;
            FTC_WAIT_DATA_WRITE WaitDataWriteComplete;
            FTH_HIGHER_OUTPUT_PINS HighPinsWriteActiveStates;
            short DataWord = 0; // short is a Signed 16-bit integer
            uint NumDataBitsToWrite = 0;
            uint NumDataBytesToWrite = 0;
            int WriteDataWordAddress;
            int ControlLocAddress1;
            int ControlLocAddress2;
            bool bWriteWait = true;
            byte[] WriteControlBuffer = new byte[WRITE_CONTROL_BUFFER_SIZE];
            byte[] WriteDataBuffer = new byte[WRITE_DATA_BUFFER_SIZE];

            FTC_INIT_CONDITION ReadStartCondition;
            int ReadDataIndex = 0;
            int ReadDataWordAddress = 0;
            byte[] ReadWordData = new byte[MAX_SPI_93LC56_CHIP_SIZE_IN_WORDS];
            FTH_HIGHER_OUTPUT_PINS HighPinsReadActiveStates;
            int ReadWordValue;
            byte[] ReadDataBuffer = new byte[READ_DATA_BUFFER_SIZE];
            uint NumDataBytesReturned = 0;
            byte[] ReadCmdSequenceDataBuffer = new byte[READ_CMDS_DATA_BUFFER_SIZE];
            short DataWordWritten = 0;
            int CharCntr = 0;
            int iLoopCntr = 0;
            string mismatchMsg = null;
            MessageBoxButtons MsgBoxButtons = MessageBoxButtons.OKCancel;
            DialogResult response = DialogResult.OK;

            byte[] byteDllVersion = new byte[MAX_NUM_DLL_VERSION_CHARS];
            string DllVersion;
            byte[] byteErrorMessage = new byte[MAX_NUM_ERROR_MESSAGE_CHARS];
            String ErrorMessage;

            this.GPIOH0InputLowHighReadStateCheckBox.Checked = false;
            this.GPIOH1InputLowHighReadStateCheckBox.Checked = false;
            this.GPIOH2InputLowHighReadStateCheckBox.Checked = false;
            this.GPIOH3InputLowHighReadStateCheckBox.Checked = false;
            this.GPIOH4InputLowHighReadStateCheckBox.Checked = false;
            this.GPIOH5InputLowHighReadStateCheckBox.Checked = false;
            this.GPIOH6InputLowHighReadStateCheckBox.Checked = false;
            this.GPIOH7InputLowHighReadStateCheckBox.Checked = false;

            // This function returns a ULONG value ie 32-bit unsigned integer
            //ftStatus = SPI_GetDllVersion(byteDllVersion, MAX_NUM_DLL_VERSION_CHARS);
            ftStatus = GetDllVersion(byteDllVersion, MAX_NUM_DLL_VERSION_CHARS);

            DllVersion = Encoding.ASCII.GetString(byteDllVersion);
            // Trim strings to first occurrence of a null terminator character
            DllVersion = DllVersion.Substring(0, DllVersion.IndexOf("\0"));

            this.DLLVersionLabel.Text = Dll_Version_Label + DllVersion;
            this.DLLVersionLabel.Refresh();

            this.DeviceNameLabel.Text = Device_Name_Label;
            this.DeviceNameLabel.Refresh();

            this.FT2232HRadioButton.Checked = false;
            this.FT4232HRadioButton.Checked = false;

            this.PassFailResultsStatusLabel.Text = "";
            this.PassFailureStatusStrip.Refresh();
            Thread.Sleep(1000);

            // The parameter of this function is a pointer to a variable of type DWORD ie 32-bit unsigned integer
            ftStatus = SPI_GetNumHiSpeedDevices(ref numHiSpeedDevices);

            if ((ftStatus == FTC_SUCCESS) && (numHiSpeedDevices > 0))
            {
                do
                {
                    ftStatus = SPI_GetHiSpeedDeviceNameLocIDChannel(hiSpeedDeviceIndex, byteHiSpeedDeviceName, MAX_NUM_DEVICE_NAME_CHARS, ref locationID, byteHiSpeedDeviceChannel, MAX_NUM_CHANNEL_CHARS, ref hiSpeedDeviceType);

                    if (ftStatus == FTC_SUCCESS)
                    {
                        hiSpeedChannel = Encoding.ASCII.GetString(byteHiSpeedDeviceChannel);
                        // Trim strings to first occurrence of a null terminator character
                        hiSpeedChannel = hiSpeedChannel.Substring(0, hiSpeedChannel.IndexOf("\0"));
                    }

                    hiSpeedDeviceIndex = hiSpeedDeviceIndex + 1;
                }
                while ((ftStatus == FTC_SUCCESS) && (hiSpeedDeviceIndex < numHiSpeedDevices) && ((hiSpeedChannel != null) && (hiSpeedChannel != this.ChannelComboBox.Text)));

                if (ftStatus == FTC_SUCCESS)
                {
                    if ((hiSpeedChannel != null) && (hiSpeedChannel != this.ChannelComboBox.Text))
                        ftStatus = FTC_DEVICE_IN_USE;
                }

                if (ftStatus == FTC_SUCCESS)
                {
                    hiSpeedDeviceName = Encoding.ASCII.GetString(byteHiSpeedDeviceName);
                    // Trim strings to first occurrence of a null terminator character
                    hiSpeedDeviceName = hiSpeedDeviceName.Substring(0, hiSpeedDeviceName.IndexOf("\0"));

                    // The ftHandle parameter is a pointer to a variable of type DWORD ie 32-bit unsigned integer
                    ftStatus = SPI_OpenHiSpeedDevice(hiSpeedDeviceName, locationID, hiSpeedChannel, ref ftHandle);

                    if (ftStatus == FTC_SUCCESS)
                    {
                        this.DeviceNameLabel.Text = Device_Name_Label + hiSpeedDeviceName;
                        this.DeviceNameLabel.Refresh();

                        ftStatus = SPI_GetHiSpeedDeviceType(ftHandle, ref hiSpeedDeviceType);

                        if (ftStatus == FTC_SUCCESS)
                        {
                            if (hiSpeedDeviceType == (uint)HI_SPEED_DEVICE_TYPES.FT4232H_DEVICE_TYPE)
                                this.FT4232HRadioButton.Checked = true;
                            else if (hiSpeedDeviceType == (uint)HI_SPEED_DEVICE_TYPES.FT2232H_DEVICE_TYPE)
                                this.FT2232HRadioButton.Checked = true;
                        }
                    }
                }

                if ((ftHandle != IntPtr.Zero) && (ftStatus == FTC_SUCCESS))
                {
                    ftStatus = SPI_InitDevice(ftHandle, MAX_FREQ_93LC56_CLOCK_DIVISOR);

                    if (ftStatus == FTC_SUCCESS)
                    {
                        if ((ftStatus = SPI_GetDeviceLatencyTimer(ftHandle, ref timerValue)) == FTC_SUCCESS)
                        {
                            if ((ftStatus = SPI_SetDeviceLatencyTimer(ftHandle, 50)) == FTC_SUCCESS)
                            {
                                ftStatus = SPI_GetDeviceLatencyTimer(ftHandle, ref timerValue);

                                ftStatus = SPI_SetDeviceLatencyTimer(ftHandle, 16);

                                ftStatus = SPI_GetDeviceLatencyTimer(ftHandle, ref timerValue);
                            }
                        }
                    }

                    if (ftStatus == FTC_SUCCESS)
                    {
                        ftStatus = TestHiSpeedDeviceClock(ftHandle);
                    }

                    if (ftStatus == FTC_SUCCESS)
                    {
                        ftStatus = SPI_SetLoopback(ftHandle, bLoopBackState);
                    }

                    if ((ftStatus == FTC_SUCCESS) && (this.CommandSequenceMenuItem.CheckState == CheckState.Unchecked))
                    {
                        ftStatus = SPI_ClearDeviceCmdSequence(ftHandle);
                    }

                    if (ftStatus == FTC_SUCCESS) {
                        // Must set the chip select disable states for all the SPI devices connected to the FT2232C dual device
                        ChipSelectsDisableStates.bADBUS3ChipSelectPinState = false;
                        ChipSelectsDisableStates.bADBUS4GPIOL1PinState = true;
                        ChipSelectsDisableStates.bADBUS5GPIOL2PinState = false;
                        ChipSelectsDisableStates.bADBUS6GPIOL3PinState = true;
                        ChipSelectsDisableStates.bADBUS7GPIOL4PinState = false;

                        // Set the 8 general purpose higher input/output pins
                        HighInputOutputPins.bPin1InputOutputState = this.GPIOH0InputOutputInitialStateCheckBox.Checked;
                        HighInputOutputPins.bPin1LowHighState = this.GPIOH0OutputLowHighInitialStateCheckBox.Checked;
                        HighInputOutputPins.bPin2InputOutputState = this.GPIOH1InputOutputInitialStateCheckBox.Checked;
                        HighInputOutputPins.bPin2LowHighState = this.GPIOH1OutputLowHighInitialStateCheckBox.Checked;
                        HighInputOutputPins.bPin3InputOutputState = this.GPIOH2InputOutputInitialStateCheckBox.Checked;
                        HighInputOutputPins.bPin3LowHighState = this.GPIOH2OutputLowHighInitialStateCheckBox.Checked;
                        HighInputOutputPins.bPin4InputOutputState = this.GPIOH3InputOutputInitialStateCheckBox.Checked;
                        HighInputOutputPins.bPin4LowHighState = this.GPIOH3OutputLowHighInitialStateCheckBox.Checked;
                        HighInputOutputPins.bPin5InputOutputState = this.GPIOH4InputOutputInitialStateCheckBox.Checked;
                        HighInputOutputPins.bPin5LowHighState = this.GPIOH4OutputLowHighInitialStateCheckBox.Checked;
                        HighInputOutputPins.bPin6InputOutputState = this.GPIOH5InputOutputInitialStateCheckBox.Checked;
                        HighInputOutputPins.bPin6LowHighState = this.GPIOH5OutputLowHighInitialStateCheckBox.Checked;
                        HighInputOutputPins.bPin7InputOutputState = this.GPIOH6InputOutputInitialStateCheckBox.Checked;
                        HighInputOutputPins.bPin7LowHighState = this.GPIOH6OutputLowHighInitialStateCheckBox.Checked;
                        HighInputOutputPins.bPin8InputOutputState = this.GPIOH7InputOutputInitialStateCheckBox.Checked;
                        HighInputOutputPins.bPin8LowHighState = this.GPIOH7OutputLowHighInitialStateCheckBox.Checked;

                        ftStatus = SPI_SetHiSpeedDeviceGPIOs(ftHandle, ref ChipSelectsDisableStates, ref HighInputOutputPins);

                        if ((ftStatus == FTC_SUCCESS) && (hiSpeedDeviceType == (uint)HI_SPEED_DEVICE_TYPES.FT2232H_DEVICE_TYPE))
                        {
                            ftStatus = SPI_GetHiSpeedDeviceGPIOs(ftHandle, out HighPinsInputData);

                            if (ftStatus == FTC_SUCCESS) {
                                if (this.GPIOH0ReadUseNotUseStateCheckBox.Checked == true)
                                {
                                    this.GPIOH0InputLowHighReadStateCheckBox.Checked = HighPinsInputData.bPin1LowHighState;
                                }
                                if (this.GPIOH1ReadUseNotUseStateCheckBox.Checked == true)
                                {
                                    this.GPIOH1InputLowHighReadStateCheckBox.Checked = HighPinsInputData.bPin2LowHighState;
                                }
                                if (this.GPIOH2ReadUseNotUseStateCheckBox.Checked == true)
                                {
                                    this.GPIOH2InputLowHighReadStateCheckBox.Checked = HighPinsInputData.bPin3LowHighState;
                                }
                                if (this.GPIOH3ReadUseNotUseStateCheckBox.Checked == true)
                                {
                                    this.GPIOH3InputLowHighReadStateCheckBox.Checked = HighPinsInputData.bPin4LowHighState;
                                }
                                if (this.GPIOH4ReadUseNotUseStateCheckBox.Checked == true)
                                {
                                    this.GPIOH4InputLowHighReadStateCheckBox.Checked = HighPinsInputData.bPin5LowHighState;
                                }
                                if (this.GPIOH5ReadUseNotUseStateCheckBox.Checked == true)
                                {
                                    this.GPIOH5InputLowHighReadStateCheckBox.Checked = HighPinsInputData.bPin6LowHighState;
                                }
                                if (this.GPIOH6ReadUseNotUseStateCheckBox.Checked == true)
                                {
                                    this.GPIOH6InputLowHighReadStateCheckBox.Checked = HighPinsInputData.bPin7LowHighState;
                                }
                                if (this.GPIOH7ReadUseNotUseStateCheckBox.Checked == true)
                                {
                                    this.GPIOH7InputLowHighReadStateCheckBox.Checked = HighPinsInputData.bPin8LowHighState;
                                }
                            }
                        }

                        // Sleep for 200 milli-seconds
                        Thread.Sleep(200);

                        WriteStartCondition.bClockPinState = false;
                        WriteStartCondition.bDataOutPinState = false;
                        WriteStartCondition.bChipSelectPinState = false;
                        WriteStartCondition.ChipSelectPin = ADBUS3ChipSelect;

                        if (ftStatus == FTC_SUCCESS) {
                            // enable writing
                            ftStatus = SPI_EWEN_EWDS_ERAL_93LC56B_Device(ftHandle, ref WriteStartCondition, SPIEWENCmdIndex);

                            if (ftStatus == FTC_SUCCESS) {
                                // erase all
                                ftStatus = SPI_EWEN_EWDS_ERAL_93LC56B_Device(ftHandle, ref WriteStartCondition, SPIERALCmdIndex);

                                if (ftStatus == FTC_SUCCESS) {
                                    // disable writing
                                    ftStatus = SPI_EWEN_EWDS_ERAL_93LC56B_Device(ftHandle, ref WriteStartCondition, SPIEWDSCmdIndex);
                                }
                            }
                        }

                        // Sleep for 200 milli-seconds
                        Thread.Sleep(200);
                       
                        if ((ftStatus == FTC_SUCCESS) && (this.GPIOH0To7OnlyCheckBox.Checked == false)) {
                            WaitDataWriteComplete.bWaitDataWriteComplete = true;
                            WaitDataWriteComplete.WaitDataWritePin = ADBUS2DataIn;
                            WaitDataWriteComplete.bDataWriteCompleteState = true;
                            WaitDataWriteComplete.DataWriteTimeoutmSecs = 5000;

                            HighPinsWriteActiveStates.bPin1State = this.GPIOH0WriteUseNotUseStateCheckBox.Checked;
                            HighPinsWriteActiveStates.bPin1ActiveState = this.GPIOH0OutputLowHighWriteStateCheckBox.Checked;
                            HighPinsWriteActiveStates.bPin2State = this.GPIOH1WriteUseNotUseStateCheckBox.Checked;
                            HighPinsWriteActiveStates.bPin2ActiveState = this.GPIOH1OutputLowHighWriteStateCheckBox.Checked;
                            HighPinsWriteActiveStates.bPin3State = this.GPIOH2WriteUseNotUseStateCheckBox.Checked;
                            HighPinsWriteActiveStates.bPin3ActiveState = this.GPIOH2OutputLowHighWriteStateCheckBox.Checked;
                            HighPinsWriteActiveStates.bPin4State = this.GPIOH3WriteUseNotUseStateCheckBox.Checked;
                            HighPinsWriteActiveStates.bPin4ActiveState = this.GPIOH3OutputLowHighWriteStateCheckBox.Checked;
                            HighPinsWriteActiveStates.bPin5State = this.GPIOH4WriteUseNotUseStateCheckBox.Checked;
                            HighPinsWriteActiveStates.bPin5ActiveState = this.GPIOH4OutputLowHighWriteStateCheckBox.Checked;
                            HighPinsWriteActiveStates.bPin6State = this.GPIOH5WriteUseNotUseStateCheckBox.Checked;
                            HighPinsWriteActiveStates.bPin6ActiveState = this.GPIOH5OutputLowHighWriteStateCheckBox.Checked;
                            HighPinsWriteActiveStates.bPin7State = this.GPIOH6WriteUseNotUseStateCheckBox.Checked;
                            HighPinsWriteActiveStates.bPin7ActiveState = this.GPIOH6OutputLowHighWriteStateCheckBox.Checked;
                            HighPinsWriteActiveStates.bPin8State = this.GPIOH7WriteUseNotUseStateCheckBox.Checked;
                            HighPinsWriteActiveStates.bPin8ActiveState = this.GPIOH7OutputLowHighWriteStateCheckBox.Checked;

                            DataWord = 0;

                            NumDataBitsToWrite = 16;
                            NumDataBytesToWrite = 2;

                            WriteDataWordAddress = 0;

                            do {
                                // set up write command and address
                                ControlLocAddress1 = 0xA0;
                                //ControlLocAddress1 = (ControlLocAddress1 || ((WriteDataWordAddress >> 3) & 0x0F))
                                ControlLocAddress1 = (ControlLocAddress1 + ((WriteDataWordAddress / 8) & 0x0F));

                                // shift left 5 bits ie make bottom 3 bits the 3 MSB's
                                ControlLocAddress2 = ((WriteDataWordAddress & 0x07) * 32);

                                WriteControlBuffer[0] = (byte)(ControlLocAddress1 & 0xFF);
                                WriteControlBuffer[1] = (byte)(ControlLocAddress2 & 0xFF);

                                // write data
                                WriteDataBuffer[0] = (byte)(DataWord & 0xFF);
                                // shift down by 8 bits
                                WriteDataBuffer[1] = (byte)((DataWord >> 8) & 0xFF);

                                ftStatus = SPI_WriteHiSpeedDevice(ftHandle, ref WriteStartCondition, true, false, 11, WriteControlBuffer, 2, bWriteWait, NumDataBitsToWrite, WriteDataBuffer, NumDataBytesToWrite, ref WaitDataWriteComplete, ref HighPinsWriteActiveStates);

                                if (DataWord != 30583) {
                                    DataWord = (short)(DataWord + 1);
                                }

                                WriteDataWordAddress = WriteDataWordAddress + 1;
                            }
                            while ((WriteDataWordAddress < MAX_SPI_93LC56_CHIP_SIZE_IN_WORDS) && (ftStatus == FTC_SUCCESS));

                            // Sleep for 200 milliseconds
                            Thread.Sleep(200);

                            if (ftStatus == FTC_SUCCESS) {
                                if (ftStatus == FTC_SUCCESS) {
                                    do {
                                        Thread.Sleep(200);

                                        ReadStartCondition.bClockPinState = false;
                                        ReadStartCondition.bDataOutPinState = false;
                                        ReadStartCondition.bChipSelectPinState = false;
                                        ReadStartCondition.ChipSelectPin = ADBUS3ChipSelect;

                                        for (ReadDataIndex = 0; ReadDataIndex < MAX_SPI_93LC56_CHIP_SIZE_IN_WORDS; ReadDataIndex++) {
                                            ReadWordData[ReadDataIndex] = 0;
                                        }

                                        ReadDataWordAddress = 0;

                                        do {
                                            // set up read command and address
                                            ControlLocAddress1 = 192;  //0xC0;
                                            //ControlLocAddress1 = (ControlLocAddress1 + ((ReadDataWordAddress >> 3) & 0xFF));
                                            ControlLocAddress1 = (ControlLocAddress1 + ((ReadDataWordAddress / 8) & 0xFF));

                                            // shift left 5 bits ie make bottom 3 bits the 3 MSB's
                                            ControlLocAddress2 = ((ReadDataWordAddress & 0x07) * 32);

                                            WriteControlBuffer[0] = (byte)(ControlLocAddress1 & 0xFF);
                                            WriteControlBuffer[1] = (byte)(ControlLocAddress2 & 0xFF);

                                            HighPinsReadActiveStates.bPin1State = this.GPIOH0ReadUseNotUseStateCheckBox.Checked;
                                            HighPinsReadActiveStates.bPin1ActiveState = false;
                                            HighPinsReadActiveStates.bPin2State = this.GPIOH1ReadUseNotUseStateCheckBox.Checked;
                                            HighPinsReadActiveStates.bPin2ActiveState = false;
                                            HighPinsReadActiveStates.bPin3State = this.GPIOH2ReadUseNotUseStateCheckBox.Checked;
                                            HighPinsReadActiveStates.bPin3ActiveState = false;
                                            HighPinsReadActiveStates.bPin4State = this.GPIOH3ReadUseNotUseStateCheckBox.Checked;
                                            HighPinsReadActiveStates.bPin4ActiveState = false;
                                            HighPinsReadActiveStates.bPin5State = this.GPIOH4ReadUseNotUseStateCheckBox.Checked;
                                            HighPinsReadActiveStates.bPin5ActiveState = false;
                                            HighPinsReadActiveStates.bPin6State = this.GPIOH5ReadUseNotUseStateCheckBox.Checked;
                                            HighPinsReadActiveStates.bPin6ActiveState = false;
                                            HighPinsReadActiveStates.bPin7State = this.GPIOH6ReadUseNotUseStateCheckBox.Checked;
                                            HighPinsReadActiveStates.bPin7ActiveState = false;
                                            HighPinsReadActiveStates.bPin8State = this.GPIOH7ReadUseNotUseStateCheckBox.Checked;
                                            HighPinsReadActiveStates.bPin8ActiveState = false;

                                            if (this.CommandSequenceMenuItem.CheckState == CheckState.Unchecked)
                                            {
                                                ftStatus = SPI_ReadHiSpeedDevice(ftHandle, ref ReadStartCondition, true, false, NUM_93LC56B_CMD_CONTOL_BITS, WriteControlBuffer, NUM_93LC56B_CMD_CONTOL_BYTES, true, false, NUM_93LC56B_CMD_DATA_BITS, ReadDataBuffer, out NumDataBytesReturned, ref HighPinsReadActiveStates);

                                                if (ftStatus == FTC_SUCCESS) {
                                                    if (HighPinsReadActiveStates.bPin1State == true) {
                                                        this.GPIOH0InputLowHighReadStateCheckBox.Checked = HighPinsReadActiveStates.bPin1ActiveState;
                                                    }
                                                    if (HighPinsReadActiveStates.bPin2State == true) {
                                                        this.GPIOH1InputLowHighReadStateCheckBox.Checked = HighPinsReadActiveStates.bPin2ActiveState;
                                                    }
                                                    if (HighPinsReadActiveStates.bPin3State == true) {
                                                        this.GPIOH2InputLowHighReadStateCheckBox.Checked = HighPinsReadActiveStates.bPin2ActiveState;
                                                    }
                                                    if (HighPinsReadActiveStates.bPin4State == true) {
                                                        this.GPIOH3InputLowHighReadStateCheckBox.Checked = HighPinsReadActiveStates.bPin4ActiveState;
                                                    }
                                                    if (HighPinsReadActiveStates.bPin5State == true)
                                                    {
                                                        this.GPIOH4InputLowHighReadStateCheckBox.Checked = HighPinsReadActiveStates.bPin5ActiveState;
                                                    }
                                                    if (HighPinsReadActiveStates.bPin6State == true)
                                                    {
                                                        this.GPIOH5InputLowHighReadStateCheckBox.Checked = HighPinsReadActiveStates.bPin6ActiveState;
                                                    }
                                                    if (HighPinsReadActiveStates.bPin7State == true)
                                                    {
                                                        this.GPIOH6InputLowHighReadStateCheckBox.Checked = HighPinsReadActiveStates.bPin7ActiveState;
                                                    }
                                                    if (HighPinsReadActiveStates.bPin8State == true)
                                                    {
                                                        this.GPIOH7InputLowHighReadStateCheckBox.Checked = HighPinsReadActiveStates.bPin8ActiveState;
                                                    }

                                                    ReadWordValue = ((ReadDataBuffer[1] * 256) + ReadDataBuffer[0]);

                                                    ReadWordData[ReadDataWordAddress] = (byte)ReadWordValue;
                                                }
                                            } else {
                                                ftStatus = SPI_AddHiSpeedDeviceReadCmd(ftHandle, ref ReadStartCondition, true, false, NUM_93LC56B_CMD_CONTOL_BITS, WriteControlBuffer, NUM_93LC56B_CMD_CONTOL_BYTES, true, false, NUM_93LC56B_CMD_DATA_BITS, ref HighPinsReadActiveStates);
                                            }

                                            ReadDataWordAddress = ReadDataWordAddress + 1;
                                        }
                                        while ((ReadDataWordAddress < MAX_SPI_93LC56_CHIP_SIZE_IN_WORDS) && (ftStatus == FTC_SUCCESS));

                                        if (ftStatus == FTC_SUCCESS) {
                                            ReadDataIndex = 0;

                                            if (this.CommandSequenceMenuItem.CheckState == CheckState.Checked)
                                            {
                                                ftStatus = SPI_ExecuteDeviceCmdSequence(ftHandle, ReadCmdSequenceDataBuffer, out NumDataBytesReturned);

                                                if (ftStatus == FTC_SUCCESS) {
                                                    if (NumDataBytesReturned > 0) {
                                                        ReadDataWordAddress = 0;

                                                        ReadDataIndex = 0;

                                                        do {
                                                            ReadWordValue = ((ReadCmdSequenceDataBuffer[ReadDataIndex + 1] * 256) + ReadCmdSequenceDataBuffer[ReadDataIndex]);

                                                            ReadWordData[ReadDataWordAddress] = (byte)ReadWordValue;

                                                            ReadDataIndex = (ReadDataIndex + 2);

                                                            ReadDataWordAddress = ReadDataWordAddress + 1;
                                                        }
                                                        while (ReadDataIndex < NumDataBytesReturned);
                                                    } else {
                                                        response = MessageBox.Show("No data returned.", "SPI Execute Command Sequence", MsgBoxButtons);
                                                    }
                                                }
                                            }

                                            if ((ftStatus == FTC_SUCCESS) && (response == DialogResult.OK))
                                            {
                                                ReadDataIndex = 0;

                                                do {
                                                    if (DataWord == 30583) {
                                                        DataWordWritten = DataWord;
                                                    } else {
                                                        DataWordWritten = (short)ReadDataIndex;
                                                    }

                                                    if (((ReadWordData[ReadDataIndex] != DataWordWritten) && (DataWord != 0)) || ((ReadWordData[ReadDataIndex] != DataWordWritten) && (DataWord == 0))) {
                                                        for (CharCntr = 0; CharCntr < 100; CharCntr++) {
                                                            WriteDataBuffer[CharCntr] = 0;
                                                        }

                                                        mismatchMsg = String.Format("SPI 93LC56 Read/Write mismatch, Address = {0}, Write Value = {1}, Read Value = {2}", ReadDataIndex, DataWordWritten, ReadWordData[ReadDataIndex]);

                                                        response = MessageBox.Show(mismatchMsg, "SPI Read Error Message", MsgBoxButtons);
                                                    }

                                                    ReadDataIndex = ReadDataIndex + 1;
                                                }
                                                while ((ReadDataIndex < MAX_SPI_93LC56_CHIP_SIZE_IN_WORDS) && (response == DialogResult.OK));
                                            }
                                        }

                                        iLoopCntr = iLoopCntr + 1;
                                    }
                                    while ((iLoopCntr < 10) && (ftStatus == FTC_SUCCESS) && (response == DialogResult.OK));
                                }
                            }
                        }
                    }
                }
            }


            if (ftHandle != IntPtr.Zero)
            {
                //CloseFinalStatePinsData.bTCKPinState = true;
                //CloseFinalStatePinsData.bTCKPinActiveState = true;
                //CloseFinalStatePinsData.bTDIPinState = true;
                //CloseFinalStatePinsData.bTDIPinActiveState = false;
                //CloseFinalStatePinsData.bTMSPinState = true;
                //CloseFinalStatePinsData.bTMSPinActiveState = true;

                //ftStatus = SPI_CloseDevice(ftHandle, ref CloseFinalStatePinsData);

                SPI_Close(ftHandle);
                ftHandle = IntPtr.Zero;
            }

            if ((ftStatus != FTC_SUCCESS) || (numHiSpeedDevices == 0) || (mismatchMsg != null))
            {
                if (ftStatus != FTC_SUCCESS)
                {
                    ErrorMessage = "Status Code(" + ftStatus.ToString() + ") - ";

                    ftStatus = SPI_GetErrorCodeString("EN", ftStatus, byteErrorMessage, MAX_NUM_ERROR_MESSAGE_CHARS);

                    ErrorMessage = ErrorMessage + Encoding.ASCII.GetString(byteErrorMessage);
                    // Trim strings to first occurrence of a null terminator character
                    ErrorMessage = ErrorMessage.Substring(0, ErrorMessage.IndexOf("\0"));

                    MessageBox.Show(ErrorMessage);
                }
                else
                {
                    if (numHiSpeedDevices == 0)
                    {
                        ErrorMessage = "There are no devices connected.";

                        MessageBox.Show(ErrorMessage);
                    }
                    else
                    {
                        ErrorMessage = mismatchMsg;
                    }
                }

                this.PassFailResultsStatusLabel.Text = "Fail - " + ErrorMessage;
                this.PassFailureStatusStrip.Refresh();

                //Console.WriteLine(ErrorMessage);
                Console.WriteLine("Fail - {0}", ErrorMessage);
            }
            else
            {
                this.PassFailResultsStatusLabel.Text = "Pass";
                this.PassFailureStatusStrip.Refresh();
            }
        }

        private void CommandSequenceMenuItem_Click(object sender, EventArgs e)
        {
            if (this.CommandSequenceMenuItem.CheckState == CheckState.Unchecked)
            {
                this.CommandSequenceMenuItem.CheckState = CheckState.Checked;
            }
            else
            {
                this.CommandSequenceMenuItem.CheckState = CheckState.Unchecked;
            }
        }

        private void ExitMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void GPIOH0InputOutputInitialStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH0InputOutputInitialStateCheckBox.Checked == false)
            {
                this.GPIOH0OutputLowHighInitialStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH0OutputLowHighInitialStateCheckBox.Enabled = true;
            }

            this.GPIOH0OutputLowHighInitialStateCheckBox.Checked = false;
        }

        private void GPIOH1InputOutputInitialStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH1InputOutputInitialStateCheckBox.Checked == false)
            {
                this.GPIOH1OutputLowHighInitialStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH1OutputLowHighInitialStateCheckBox.Enabled = true;
            }

            this.GPIOH1OutputLowHighInitialStateCheckBox.Checked = false;
        }

        private void GPIOH2InputOutputInitialStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH2InputOutputInitialStateCheckBox.Checked == false)
            {
                this.GPIOH2OutputLowHighInitialStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH2OutputLowHighInitialStateCheckBox.Enabled = true;
            }

            this.GPIOH2OutputLowHighInitialStateCheckBox.Checked = false;
        }

        private void GPIOH3InputOutputInitialStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH3InputOutputInitialStateCheckBox.Checked == false)
            {
                this.GPIOH3OutputLowHighInitialStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH3OutputLowHighInitialStateCheckBox.Enabled = true;
            }

            this.GPIOH3OutputLowHighInitialStateCheckBox.Checked = false;
        }

        private void GPIOH4InputOutputInitialStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH4InputOutputInitialStateCheckBox.Checked == false)
            {
                this.GPIOH4OutputLowHighInitialStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH4OutputLowHighInitialStateCheckBox.Enabled = true;
            }

            this.GPIOH4OutputLowHighInitialStateCheckBox.Checked = false;
        }

        private void GPIOH5InputOutputInitialStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH5InputOutputInitialStateCheckBox.Checked == false)
            {
                this.GPIOH5OutputLowHighInitialStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH5OutputLowHighInitialStateCheckBox.Enabled = true;
            }

            this.GPIOH5OutputLowHighInitialStateCheckBox.Checked = false;
        }

        private void GPIOH6InputOutputInitialStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH6InputOutputInitialStateCheckBox.Checked == false)
            {
                this.GPIOH6OutputLowHighInitialStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH6OutputLowHighInitialStateCheckBox.Enabled = true;
            }

            this.GPIOH6OutputLowHighInitialStateCheckBox.Checked = false;
        }

        private void GPIOH7InputOutputInitialStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH7InputOutputInitialStateCheckBox.Checked == false)
            {
                this.GPIOH7OutputLowHighInitialStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH7OutputLowHighInitialStateCheckBox.Enabled = true;
            }

            this.GPIOH7OutputLowHighInitialStateCheckBox.Checked = false;
        }

        private void GPIOH0WriteUseNotUseStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH0WriteUseNotUseStateCheckBox.Checked == false) {
                this.GPIOH0OutputLowHighWriteStateCheckBox.Enabled = false;
            } else {
                this.GPIOH0OutputLowHighWriteStateCheckBox.Enabled = true;
            }

            this.GPIOH0OutputLowHighWriteStateCheckBox.Checked = false;
        }

        private void GPIOH1WriteUseNotUseStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH1WriteUseNotUseStateCheckBox.Checked == false)
            {
                this.GPIOH1OutputLowHighWriteStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH1OutputLowHighWriteStateCheckBox.Enabled = true;
            }

            this.GPIOH1OutputLowHighWriteStateCheckBox.Checked = false;
        }

        private void GPIOH2WriteUseNotUseStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH2WriteUseNotUseStateCheckBox.Checked == false)
            {
                this.GPIOH2OutputLowHighWriteStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH2OutputLowHighWriteStateCheckBox.Enabled = true;
            }

            this.GPIOH2OutputLowHighWriteStateCheckBox.Checked = false;
        }

        private void GPIOH3WriteUseNotUseStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH3WriteUseNotUseStateCheckBox.Checked == false)
            {
                this.GPIOH3OutputLowHighWriteStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH3OutputLowHighWriteStateCheckBox.Enabled = true;
            }

            this.GPIOH3OutputLowHighWriteStateCheckBox.Checked = false;
        }

        private void GPIOH4WriteUseNotUseStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH4WriteUseNotUseStateCheckBox.Checked == false)
            {
                this.GPIOH4OutputLowHighWriteStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH4OutputLowHighWriteStateCheckBox.Enabled = true;
            }

            this.GPIOH4OutputLowHighWriteStateCheckBox.Checked = false;
        }

        private void GPIOH5WriteUseNotUseStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH5WriteUseNotUseStateCheckBox.Checked == false)
            {
                this.GPIOH5OutputLowHighWriteStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH5OutputLowHighWriteStateCheckBox.Enabled = true;
            }

            this.GPIOH5OutputLowHighWriteStateCheckBox.Checked = false;
        }

        private void GPIOH6WriteUseNotUseStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH6WriteUseNotUseStateCheckBox.Checked == false)
            {
                this.GPIOH6OutputLowHighWriteStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH6OutputLowHighWriteStateCheckBox.Enabled = true;
            }

            this.GPIOH6OutputLowHighWriteStateCheckBox.Checked = false;
        }

        private void GPIOH7WriteUseNotUseStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH7WriteUseNotUseStateCheckBox.Checked == false)
            {
                this.GPIOH7OutputLowHighWriteStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH7OutputLowHighWriteStateCheckBox.Enabled = true;
            }

            this.GPIOH7OutputLowHighWriteStateCheckBox.Checked = false;
        }

        private void GPIOH0ReadUseNotUseStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH0ReadUseNotUseStateCheckBox.Checked == false) {
                this.GPIOH0InputLowHighReadStateCheckBox.Enabled = false;
            } else {
                this.GPIOH0InputLowHighReadStateCheckBox.Enabled = true;
            }

            this.GPIOH0InputLowHighReadStateCheckBox.Checked = false;
        }

        private void GPIOH1ReadUseNotUseStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH1ReadUseNotUseStateCheckBox.Checked == false)
            {
                this.GPIOH1InputLowHighReadStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH1InputLowHighReadStateCheckBox.Enabled = true;
            }

            this.GPIOH1InputLowHighReadStateCheckBox.Checked = false;
        }

        private void GPIOH2ReadUseNotUseStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH2ReadUseNotUseStateCheckBox.Checked == false)
            {
                this.GPIOH2InputLowHighReadStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH2InputLowHighReadStateCheckBox.Enabled = true;
            }

            this.GPIOH2InputLowHighReadStateCheckBox.Checked = false;
        }

        private void GPIOH3ReadUseNotUseStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH3ReadUseNotUseStateCheckBox.Checked == false)
            {
                this.GPIOH3InputLowHighReadStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH3InputLowHighReadStateCheckBox.Enabled = true;
            }

            this.GPIOH3InputLowHighReadStateCheckBox.Checked = false;
        }

        private void GPIOH4ReadUseNotUseStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH4ReadUseNotUseStateCheckBox.Checked == false)
            {
                this.GPIOH4InputLowHighReadStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH4InputLowHighReadStateCheckBox.Enabled = true;
            }

            this.GPIOH4InputLowHighReadStateCheckBox.Checked = false;
        }

        private void GPIOH5ReadUseNotUseStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH5ReadUseNotUseStateCheckBox.Checked == false)
            {
                this.GPIOH5InputLowHighReadStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH5InputLowHighReadStateCheckBox.Enabled = true;
            }

            this.GPIOH5InputLowHighReadStateCheckBox.Checked = false;
        }

        private void GPIOH6ReadUseNotUseStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH6ReadUseNotUseStateCheckBox.Checked == false)
            {
                this.GPIOH6InputLowHighReadStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH6InputLowHighReadStateCheckBox.Enabled = true;
            }

            this.GPIOH6InputLowHighReadStateCheckBox.Checked = false;
        }

        private void GPIOH7ReadUseNotUseStateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.GPIOH7ReadUseNotUseStateCheckBox.Checked == false)
            {
                this.GPIOH7InputLowHighReadStateCheckBox.Enabled = false;
            }
            else
            {
                this.GPIOH7InputLowHighReadStateCheckBox.Enabled = true;
            }

            this.GPIOH7InputLowHighReadStateCheckBox.Checked = false;
        }
    }
}